@@ -10,7 +10,8 @@ from django.core.urlresolvers import reverse
 from django.core.exceptions import ValidationError
 from django.test import TestCase
 from django.test.client import Client
-from pybb.templatetags.pybb_tags import pybb_is_topic_unread, pybb_topic_unread, pybb_forum_unread
+from pybb.templatetags.pybb_tags import pybb_is_topic_unread, pybb_topic_unread, pybb_forum_unread, \
+     pybb_get_latest_topics, pybb_get_latest_posts
 
 from pybb import util
 User = util.get_user_model()
@@ -36,15 +37,11 @@ class SharedTestModule(object):
         self.client.login(username=username, password=password)
 
     def create_initial(self, post=True):
-        self.category = Category(name='foo')
-        self.category.save()
-        self.forum = Forum(name='xfoo', description='bar', category=self.category)
-        self.forum.save()
-        self.topic = Topic(name='etopic', forum=self.forum, user=self.user)
-        self.topic.save()
+        self.category = Category.objects.create(name='foo')
+        self.forum = Forum.objects.create(name='xfoo', description='bar', category=self.category)
+        self.topic = Topic.objects.create(name='etopic', forum=self.forum, user=self.user)
         if post:
-            self.post = Post(topic=self.topic, user=self.user, body='bbcode [b]test[b]')
-            self.post.save()
+            self.post = Post.objects.create(topic=self.topic, user=self.user, body='bbcode [b]test[b]')
 
     def get_form_values(self, response, form="post-form"):
         return dict(html.fromstring(response.content).xpath('//form[@class="%s"]' % form)[0].form_values())
@@ -657,12 +654,29 @@ class FeaturesTest(TestCase, SharedTestModule):
 
     def test_user_blocking(self):
         user = User.objects.create_user('test', 'test@localhost', 'test')
+        topic = Topic.objects.create(name='topic', forum=self.forum, user=self.user)
+        self.post = Post.objects.create(topic=topic, user=user, body='bbcode [b]test[b]')
         self.user.is_superuser = True
         self.user.save()
         self.login_client()
-        self.assertEqual(self.client.get(reverse('pybb:block_user', args=[user.username]), follow=True).status_code, 200)
+        response = self.client.get(reverse('pybb:block_user', args=[user.username]), follow=True)
+        self.assertEqual(response.status_code, 405)
+        response = self.client.post(reverse('pybb:block_user', args=[user.username]), follow=True)
+        self.assertEqual(response.status_code, 200)
         user = User.objects.get(username=user.username)
         self.assertFalse(user.is_active)
+        self.assertEqual(Topic.objects.filter().count(), 2)
+        self.assertEqual(Post.objects.filter(user=user).count(), 1)
+
+        user.is_active = True
+        user.save()
+        response = self.client.post(reverse('pybb:block_user', args=[user.username]),
+                                    data={'block_and_delete_messages': 'block_and_delete_messages'}, follow=True)
+        self.assertEqual(response.status_code, 200)
+        user = User.objects.get(username=user.username)
+        self.assertFalse(user.is_active)
+        self.assertEqual(Topic.objects.filter().count(), 1)
+        self.assertEqual(Post.objects.filter(user=user).count(), 0)
 
     def test_ajax_preview(self):
         self.login_client()
@@ -827,9 +841,28 @@ class FeaturesTest(TestCase, SharedTestModule):
         self.assertEqual(forum_1.topic_count, 0)
         self.assertEqual(forum_1.post_count, 0)
 
-    def test_user_view(self):
-        resp = self.client.get(reverse('pybb:user', kwargs={'username': self.user.username}))
-        self.assertEqual(resp.status_code, 200)
+    def test_user_views(self):
+        response = self.client.get(reverse('pybb:user', kwargs={'username': self.user.username}))
+        self.assertEqual(response.status_code, 200)
+
+        response = self.client.get(reverse('pybb:user_posts', kwargs={'username': self.user.username}))
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.context['object_list'].count(), 1)
+
+        response = self.client.get(reverse('pybb:user_topics', kwargs={'username': self.user.username}))
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.context['object_list'].count(), 1)
+
+        self.topic.forum.hidden = True
+        self.topic.forum.save()
+
+        self.client.logout()
+
+        response = self.client.get(reverse('pybb:user_posts', kwargs={'username': self.user.username}))
+        self.assertEqual(response.context['object_list'].count(), 0)
+
+        response = self.client.get(reverse('pybb:user_topics', kwargs={'username': self.user.username}))
+        self.assertEqual(response.context['object_list'].count(), 0)
 
     def test_post_count(self):
         topic = Topic(name='etopic', forum=self.forum, user=self.user)
@@ -843,6 +876,26 @@ class FeaturesTest(TestCase, SharedTestModule):
         post.delete()
         self.assertEqual(Profile.objects.get(pk=util.get_pybb_profile(self.user).pk).post_count, 1)
 
+    def test_latest_topics_tag(self):
+        Topic.objects.all().delete()
+        for i in range(10):
+            Topic.objects.create(name='topic%s' % i, user = self.user, forum=self.forum)
+        latest_topics = pybb_get_latest_topics(context=None, user=self.user)
+        self.assertEqual(len(latest_topics), 5)
+        self.assertEqual(latest_topics[0].name, 'topic9')
+        self.assertEqual(latest_topics[4].name, 'topic5')
+
+
+    def test_latest_posts_tag(self):
+        Post.objects.all().delete()
+        for i in range(10):
+            Post.objects.create(body='post%s' % i, user = self.user, topic=self.topic)
+        latest_topics = pybb_get_latest_posts(context=None, user=self.user)
+        self.assertEqual(len(latest_topics), 5)
+        self.assertEqual(latest_topics[0].body, 'post9')
+        self.assertEqual(latest_topics[4].body, 'post5')
+
+
     def tearDown(self):
         defaults.PYBB_ENABLE_ANONYMOUS_POST = self.ORIG_PYBB_ENABLE_ANONYMOUS_POST
         defaults.PYBB_PREMODERATION = self.ORIG_PYBB_PREMODERATION
@@ -1102,6 +1155,7 @@ class PollTest(TestCase, SharedTestModule):
         values['poll_answers-1-DELETE'] = 'on'
         values['poll_answers-TOTAL_FORMS'] = 2
         response = self.client.post(add_topic_url, values, follow=True)
+        self.assertEqual(response.status_code, 200)
         self.assertFalse(Topic.objects.filter(name='test poll name').exists())
 
     def test_regression_poll_deletion_after_second_post(self):
@@ -1192,14 +1246,28 @@ class PollTest(TestCase, SharedTestModule):
 
         # already voted
         response = self.client.post(vote_url, data=values, follow=True)
-        self.assertEqual(response.status_code, 400) # bad request status
+        self.assertEqual(response.status_code, 403) # bad request status
 
         recreate_poll(poll_type=Topic.POLL_TYPE_MULTIPLE)
         values = {'answers': [a.id for a in PollAnswer.objects.all()]}
         response = self.client.post(vote_url, data=values, follow=True)
         self.assertEqual(response.status_code, 200)
-        self.assertListEqual([a.votes() for a in PollAnswer.objects.all()], [1, 1, ])
-        self.assertListEqual([a.votes_percent() for a in PollAnswer.objects.all()], [50.0, 50.0, ])
+        self.assertListEqual([a.votes() for a in PollAnswer.objects.all()], [1, 1])
+        self.assertListEqual([a.votes_percent() for a in PollAnswer.objects.all()], [50.0, 50.0])
+
+        response = self.client.post(vote_url, data=values, follow=True)
+        self.assertEqual(response.status_code, 403)  # already voted
+
+        cancel_vote_url = reverse('pybb:topic_cancel_poll_vote', kwargs={'pk': self.topic.id})
+        response = self.client.post(cancel_vote_url, data=values, follow=True)
+        self.assertEqual(response.status_code, 200)
+        self.assertListEqual([a.votes() for a in PollAnswer.objects.all()], [0, 0])
+        self.assertListEqual([a.votes_percent() for a in PollAnswer.objects.all()], [0, 0])
+
+        response = self.client.post(vote_url, data=values, follow=True)
+        self.assertEqual(response.status_code, 200)
+        self.assertListEqual([a.votes() for a in PollAnswer.objects.all()], [1, 1])
+        self.assertListEqual([a.votes_percent() for a in PollAnswer.objects.all()], [50.0, 50.0])
 
     def tearDown(self):
         defaults.PYBB_POLL_MAX_ANSWERS = self.PYBB_POLL_MAX_ANSWERS
