@@ -7,11 +7,13 @@ from django.core.exceptions import ObjectDoesNotExist, PermissionDenied
 from django.core.urlresolvers import reverse
 from django.contrib import messages
 from django.db.models import F, Q
+from django.db.models.aggregates import Count
 from django.http import HttpResponseRedirect, HttpResponse, Http404, HttpResponseBadRequest,\
     HttpResponseForbidden
 from django.shortcuts import get_object_or_404, redirect, render
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import ugettext as _
 from django.utils.decorators import method_decorator
+from django.views.decorators.http import require_POST
 from django.views.generic.edit import ModelFormMixin
 from django.views.decorators.csrf import csrf_protect
 from django.views import generic
@@ -27,7 +29,7 @@ except ImportError:
     Page.pages = lambda self: [PageRepr(i) for i in range(1, self.paginator.num_pages + 1)]
 
 from pybb.models import Category, Forum, Topic, Post, TopicReadTracker, ForumReadTracker, PollAnswerUser
-from pybb.forms import PostForm, AdminPostForm, EditProfileForm, AttachmentFormSet, PollAnswerFormSet, PollForm
+from pybb.forms import PostForm, AdminPostForm, AttachmentFormSet, PollAnswerFormSet, PollForm
 from pybb.templatetags.pybb_tags import pybb_topic_poll_not_voted
 from pybb import defaults
 
@@ -389,6 +391,54 @@ class UserView(generic.DetailView):
         return ctx
 
 
+class UserPosts(generic.ListView):
+    model = Post
+    paginate_by = defaults.PYBB_TOPIC_PAGE_SIZE
+    paginator_class = Paginator
+    template_name = 'pybb/user_posts.html'
+
+    def dispatch(self, request, *args, **kwargs):
+        username = kwargs.pop('username')
+        self.user = get_object_or_404(**{'klass': User, username_field: username})
+        return super(UserPosts, self).dispatch(request, *args, **kwargs)
+
+    def get_queryset(self):
+        qs = super(UserPosts, self).get_queryset()
+        qs = qs.filter(user=self.user)
+        qs = perms.filter_posts(self.request.user, qs).select_related('topic')
+        qs = qs.order_by('-created', '-updated')
+        return qs
+
+    def get_context_data(self, **kwargs):
+        context = super(UserPosts, self).get_context_data(**kwargs)
+        context['target_user'] = self.user
+        return context
+
+
+class UserTopics(generic.ListView):
+    model = Topic
+    paginate_by = defaults.PYBB_FORUM_PAGE_SIZE
+    paginator_class = Paginator
+    template_name = 'pybb/user_topics.html'
+
+    def dispatch(self, request, *args, **kwargs):
+        username = kwargs.pop('username')
+        self.user = get_object_or_404(User, username=username)
+        return super(UserTopics, self).dispatch(request, *args, **kwargs)
+
+    def get_queryset(self):
+        qs = super(UserTopics, self).get_queryset()
+        qs = qs.filter(user=self.user)
+        qs = perms.filter_topics(self.user, qs)
+        qs = qs.order_by('-updated', '-created')
+        return qs
+
+    def get_context_data(self, **kwargs):
+        context = super(UserTopics, self).get_context_data(**kwargs)
+        context['target_user'] = self.user
+        return context
+
+
 class PostView(RedirectToLoginMixin, generic.RedirectView):
 
     def get_login_redirect_url(self):
@@ -416,11 +466,17 @@ class ModeratePost(generic.RedirectView):
 class ProfileEditView(generic.UpdateView):
 
     template_name = 'pybb/edit_profile.html'
-    form_class = EditProfileForm
 
     def get_object(self, queryset=None):
         return util.get_pybb_profile(self.request.user)
 
+    def get_form_class(self):
+        if not self.form_class:
+            from pybb.forms import EditProfileForm
+            return EditProfileForm
+        else:
+            return super(ProfileEditView, self).get_form_class()
+
     @method_decorator(login_required)
     @method_decorator(csrf_protect)
     def dispatch(self, request, *args, **kwargs):
@@ -530,7 +586,7 @@ class TopicPollVoteView(generic.UpdateView):
     def form_valid(self, form):
         # already voted
         if not pybb_topic_poll_not_voted(self.object, self.request.user):
-            return HttpResponseBadRequest()
+            return HttpResponseForbidden()
 
         answers = form.cleaned_data['answers']
         for answer in answers:
@@ -548,6 +604,13 @@ class TopicPollVoteView(generic.UpdateView):
         return self.object.get_absolute_url()
 
 
+@login_required
+def topic_cancel_poll_vote(request, pk):
+    topic = get_object_or_404(Topic, pk=pk)
+    PollAnswerUser.objects.filter(user=request.user, poll_answer__topic_id=topic.id).delete()
+    return HttpResponseRedirect(topic.get_absolute_url())
+
+
 @login_required
 def delete_subscription(request, topic_id):
     topic = get_object_or_404(perms.filter_topics(request.user, Topic.objects.all()), pk=topic_id)
@@ -581,12 +644,20 @@ def mark_all_as_read(request):
 
 
 @login_required
+@require_POST
 def block_user(request, username):
     user = get_object_or_404(User, **{username_field: username})
     if not perms.may_block_user(request.user, user):
         raise PermissionDenied
     user.is_active = False
     user.save()
+    if 'block_and_delete_messages' in request.POST:
+        # individually delete each post and empty topic to fire method
+        # with forum/topic counters recalculation
+        for p in Post.objects.filter(user=user):
+            p.delete()
+        for t in Topic.objects.annotate(cnt=Count('posts')).filter(cnt=0):
+            t.delete()
     msg = _('User successfuly blocked')
     messages.success(request, msg, fail_silently=True)
     return redirect('pybb:index')
